function love.run()
  _r = math.random
  _f = math.floor
  _ce = math.ceil
  _c = math.cos
  _s = math.sin
  _sq = math.sqrt
  _at2 = math.atan2
  _d2r = math.rad
  _r2d = math.deg
  _m = math.mod
  pi = math.pi
  gr = love.graphics
  ti = love.timer
  ms = love.mouse
  kb = love.keyboard
  math.randomseed(os.time())

	love.load(arg)
	while true do
	  fps = ti.getFPS()
	  dt = ti.getDelta()
		ti.step()
		time = ti.getTime()
		gr.clear()
		love.update(dt)
		love.draw(dt)
		if love.event then
			for e,a,b,c in love.event.poll() do
				if e == "q" then
					if love.audio then love.audio.stop() end
				return
				end
				love.handlers[e](a,b,c)
			end
		end
		ti.sleep(1)
		gr.present()
	end
end

function love.load()
	tiles = gr.newImage("tiles.png")

	tileQuad = {}
	for x=0,7 do
		tileQuad[x] = gr.newQuad(x * 16, 0 * 16, 16, 16, tiles:getWidth(), tiles:getHeight())
	end
  helpQuad = gr.newQuad(0, 16, 42, 28, tiles:getWidth(), tiles:getHeight())

	playerImg = gr.newImage("player.png")

	playerQuad = {}
	for x=0,1 do
    playerQuad[x] = {}
    for y=0,1 do
  		playerQuad[x][y] = gr.newQuad(x * 16, y * 16, 16, 16, playerImg:getWidth(), playerImg:getHeight())
    end
	end


  color = {}
  color[3] = {196,207,161}
  color[2] = {139,149,109}
  color[1] = {107,115,83}
  color[0] = {37,41,29}

	font = gr.newImageFont("font.png", "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789 :-xtc!", 0)

  inGame = 1
  inMenu = 0
  inTran = 2

  gr.setFont(font)

  gamemode = inMenu
end

function setupGame()
  resetIntro()

  enemy = {}
  trap = {}

	player = {
	  x, y,
	  step_x = 0, step_y = 0, step_f = 0,
	  facing = 0, lives = 3, onfloor = false, falldistance = 0, rotate = 0,
	  dying = false, dying_f = 0, level = 1,
	  jumping = false, jump_f = 0, deathreason = ""
	}

  loadLevel(1)

	waitForRelease = 0
end

function createTrap(t, x, y, w, h, s, img, quad)
  local n = #trap+1
  if t == 1 then
    trap[n] = {x = x, y = y, w = w, h = h, speed = s, shaketime = 0, type = t, active = false, img = img, quad = quad }
  end
end

function resetIntro()
  intro = { screen = {}, drip = {}, dripSpeed = {}, done = 0 }
	for x=0,19 do
	  intro.screen[x] = {}
  	for y=0,16 do
  	  intro.screen[x][y] = 3
    end
    intro.drip[x] = -2
    intro.dripSpeed[x] = _r(10,14)
	end
end

function loadLevel(l)
  level = {
    hit = {}, tile = {}, ladder = {}, time = 90, start = ti.getTime(), finished = false,
    exit = { x = 16, y = 15, animating = false, frame = 1, frametime = 0 }
  }
	for x=0,19 do
	  level.hit[x] = {}
	  level.tile[x] = {}
	  level.ladder[x] = {}
  	for y=0,17 do
  	  level.hit[x][y] = 1
  	  level.tile[x][y] = 0
  	  level.ladder[x][y] = 0
    end
	end

	for x=1,18 do
  	for y=1,15 do
  	  level.hit[x][y] = 0
  	  level.tile[x][y] = 3
    end
	end

	for i=1,20 do
  	level.tile[_r(1,17)][_r(1,14)] = 2
	end

	level.tile[level.exit.x][level.exit.y] = 5
	level.tile[2][0] = 3
	level.hit[2][0] = 0

	for i=1,2 do
	  level.hit[i][3] = 1
	  level.tile[i][3] = 1
	end

	for i=1,4 do
	  level.hit[i][5] = 1
	  level.tile[i][5] = 1
	end
	for i=3,6 do
	  level.hit[i][8] = 1
	  level.tile[i][8] = 1
	end
	for i=8,11 do
	  level.hit[i][8] = 1
	  level.tile[i][8] = 1
	end
	for i=11,13 do
	  level.hit[i][11] = 1
	  level.tile[i][11] = 1
	end

	for i=10,11 do
	  level.hit[i][13] = 1
	  level.tile[i][13] = 1
	end

  level.hit[3][16] = 0
  level.tile[3][16] = 4
  level.hit[1][16] = 0
  level.tile[1][16] = 4

  player.x = 2
  player.y = 0

  createTrap(1, 10*16, 1*16, 16, 16, 128, tiles, tileQuad[7])
  createTrap(1, 9*16, 1*16, 16, 16, 128, tiles, tileQuad[7])

  createTrap(1, 9*16, 9*16, 16, 16, 128, tiles, tileQuad[7])
end

function love.update(dt)
  local kSpace = kb.isDown(" ")
  local kLeft = kb.isDown("left") or kb.isDown("a")
  local kRight = kb.isDown("right") or kb.isDown("d")
  local kShift = kb.isDown("lshift") or kb.isDown("rshift")

  if gamemode == inTran then
    intro.done = 0
    for i=0,19 do
      intro.drip[i] = intro.drip[i] + intro.dripSpeed[i] * dt
      if intro.drip[i] >= 20 then intro.drip[i] = 20 end
      if intro.drip[i] >= 0 and intro.drip[i] <= 17 then intro.screen[i][_f(intro.drip[i])] = level.hit[i][_f(intro.drip[i])] end
      if intro.drip[i] == 20 then intro.done = intro.done + 1 end
    end
    if intro.done >= 20 then
      gamemode = inGame
    end

  elseif gamemode == inGame then

    if player.dying == false and level.finished == false then
      level.time = level.time - dt
      if level.time < 0 then
        level.time = 0
        killPlayer("      TIME UP!       ")
      end
    end

    if level.hit[_f(player.x)][_f(player.y)+1] == 0 then
      player.onfloor = false
    end

    if player.dying == false then
      if player.onfloor == false and player.step_x == 0 then
        player.step_f = 1
        player.step_y = player.step_y + 64 * dt
        if player.step_y > 8 then
          player.y = player.y + 1
          player.step_y = 0
          player.falldistance = player.falldistance + 1
          if player.falldistance > 3 then
            player.rotate = player.rotate + 45
            if player.rotate > 180 then player.rotate = 180 end
          end
          if level.hit[_f(player.x)][_f(player.y)+1] == 1 then
            player.onfloor = true
            player.step_f = 0
            if player.falldistance > 3 then
              killPlayer("       SPLAT!       ")
            elseif player.y == 16 then
              killPlayer("      IMPALED!      ")
            else
              player.falldistance = 0
            end
          end
        end
      end

      if player.onfloor == true and player.jumping == false then
        if kLeft and kRight == false then
          if player.facing == 0 then
            player.facing = 1
            waitForRelease = .1
          elseif waitForRelease == 0 then
            if player.step_x == 0 and level.hit[player.x-1][player.y] == 0 then
              player.x = player.x - 1
              player.step_x = 8
            end
          end
        elseif kRight and kLeft == false then
          if player.facing == 1 then
            player.facing = 0
            waitForRelease = .1
          elseif waitForRelease == 0 then
            if player.step_x == 0 and level.hit[player.x+1][player.y] == 0 then
              player.x = player.x + 1
              player.step_x = -8
            end
          end
        end
      end
      if kSpace and player.jumping == false and player.onfloor == true then
        if (player.facing == 0 and level.hit[player.x+1][player.y] == 0 and level.hit[player.x+1][player.y+1] == 0 and level.hit[player.x+2][player.y] == 0) or
          (player.facing == 1 and level.hit[player.x-1][player.y] == 0 and level.hit[player.x-1][player.y+1] == 0 and level.hit[player.x-2][player.y] == 0) then
          player.jumping = true
          player.jump_f = 16
        end
      end

      waitForRelease = waitForRelease - dt
      if waitForRelease < 0 then
        waitForRelease = 0
      end

      if player.jumping then
        player.step_f = 1
        player.step_y = -1
        player.jump_f = player.jump_f - 32 * dt
        if player.facing == 0 then
          player.step_x = 16-player.jump_f
        elseif player.facing == 1 then
          player.step_x = player.jump_f-16
        end
        if player.jump_f < 0 then
          player.jump_f = 0
          player.jumping = false
          if player.facing == 0 then
            player.x = player.x + 2
          else
            player.x = player.x - 2
          end
          player.step_x = 0
          player.step_y = 0
          player.step_f = 0
          playerReachedTile()
        end
      else
        if player.step_x > 0 then
          player.step_x = player.step_x - 32 * dt
          if player.step_x < 0 then player.step_x = 0 playerReachedTile() end
        elseif player.step_x < 0 then
          player.step_x = player.step_x + 32 * dt
          if player.step_x > 0 then player.step_x = 0 playerReachedTile() end
        end
      end


      if player.step_x ~= 0 then
        player.step_f = player.step_f + 8 * dt
        if player.step_f >= 2 then player.step_f = 0 end
      end
    else
      player.dying_f = player.dying_f - dt
      player.rotate = player.rotate + 720 * dt
      if player.dying_f < 0 then
        player.dying_f = 0
        player.dying = false
        player.x = 2
        player.y = 0
        player.rotate = 0
        player.jumping = false
        player.facing = 0
        level.time = 90
        if player.lives <= 0 then
          gamemode = inMenu
          --setupGame()
        end
      end
    end

    if level.exit.animating == true then
      level.exit.frametime = level.exit.frametime - dt
      if level.exit.frametime < 0 then
        level.exit.frametime = .05
        level.exit.frame = level.exit.frame + 1
        for x = level.exit.x-level.exit.frame, level.exit.x+level.exit.frame do
          for y = level.exit.y-level.exit.frame, level.exit.y+level.exit.frame do
            if x < 20 and x > -1 and y < 17 and y > -1 then
              level.tile[x][y] = 3
            end
          end
        end
        if level.exit.frame > 20 then
          gamemode = inTran
          player.level = player.level + 1
          resetIntro()
          loadLevel(player.level)
        end
      end
    end
  end
end

function playerReachedTile()
  player.step_f = 0
  local xx
  if player.facing == 0 then xx = player.x+1 else xx = player.x-1 end
  if level.hit[xx][player.y+1] == 0 then
    waitForRelease = .1
  end
  if player.x == level.exit.x and player.y == level.exit.y then
    level.exit.animating = true
    level.exit.frame = 0
    level.exit.frametime = .1
    level.finished = true
  end

  for i, t in pairs(trap) do
    if player.x >= _f(t.x/16)-1 and player.x <= _f(t.x/16)+1 and player.y >= _f(t.y/16) and t.active == false then
      t.active = true
      t.shaketime = .3
    end
  end
end

function killPlayer(dr)
  player.deathreason = dr
  player.lives = player.lives - 1
  player.dying = true
  player.dying_f = 2
  player.falldistance = 0
end

function love.draw()

  if gamemode == inMenu then
    gr.setBackgroundColor(unpack(color[3]))
    gr.setColor(unpack(color[0]))
    gr.print("\n\n\n\n\n\n\n\n\n\n\n\n    PRESS  SPACE\n\n      TO  PLAY\n\n\n    c2010 JASOCO", 0, -16)
  elseif gamemode == inTran then
    gr.setBackgroundColor(unpack(color[3]))
    gr.setColor(255,255,255)
    drawLevel()
    for i=0,19 do
      gr.setColor(unpack(color[0]))
      gr.rectangle("fill", i*16-2, intro.drip[i]*16-2, 20, 400)
    end
    for i=0,19 do
      gr.setColor(unpack(color[3]))
      gr.rectangle("fill", i*16, intro.drip[i]*16, 16, 400)
    end
  elseif gamemode == inGame then
    gr.setBackgroundColor(unpack(color[3]))
    gr.setColor(255,255,255)


    drawLevel()

    gr.drawq(playerImg, playerQuad[player.facing][_f(player.step_f)], (player.x*8 + player.step_x)*2+8, (player.y*8 + player.step_y)*2+8, _d2r(_f(player.rotate/90)*90), 1, 1, 8, 8)

    for i, t in pairs(trap) do
      if t.active then
        t.shaketime = t.shaketime - dt
        if t.shaketime < 0 then
          t.shaketime = 0
          t.y = t.y + t.speed * dt
        end
        if _f(t.x/16) == player.x and _f(t.y/16) == player.y then
          local tt
          if t.type == 1 then tt = "    THAT SMARTS!    " end
          killPlayer(tt)
        end
      end
      gr.drawq(t.img, t.quad, t.x+(_c(t.shaketime*100)), t.y)
    end

    local helpTick = _c(level.time*4)
    if helpTick > 0 and level.finished == false then
      gr.drawq(tiles, helpQuad, (level.exit.x*16) - 30, (level.exit.y*16) - 30)
    end

    drawStatusBar()

    if player.dying then
      local t = 8*16
      if player.y == 8 or player.y == 9 then t = 11*16 end
      gr.setColor(unpack(color[3]))
      gr.rectangle("fill", 0, t+8, 320, 32)
      gr.setColor(unpack(color[0]))
      gr.print(player.deathreason, 0, t)
    end
  end

--  gr.setColor(unpack(color[3]))
--  t = "FPS:" .. fps
--  gr.print(t, 0, -16)
end

function drawStatusBar()
  gr.setColor(unpack(color[3]))
  gr.rectangle("fill", 0, 288-16, 320, 16)
  gr.setColor(unpack(color[0]))
  gr.print("  x"..player.lives.."  LV:"..player.level.."  t".._ce(level.time), 0, 8*16*2)
  gr.setColor(255,255,255)
  gr.drawq(playerImg, playerQuad[0][0], 1*16, 17*16)
end

function drawLevel()
  for x=0,19 do
    for y=0,16 do
      gr.drawq(tiles, tileQuad[level.tile[x][y]], (x*8)*2, (y*8)*2)
    end
  end
end

function love.mousepressed(x, y, button)
  if button == "l" then

  end
end

function love.keypressed(k)
  if k == " " then
    if gamemode == inMenu then
      gamemode = inTran
      setupGame()
    end
  elseif k == "x" then
    local x, y = _r(1,18), 0
      if level.hit[x][y] == 1 then createTrap(1, x*16, y, 16, 16, 128, tiles, tileQuad[level.tile[x][y]]) end
      level.tile[x][y] = 3
      level.hit[x][y] = 0
  end
end

function degToRad(d) return d * pi / 180 end

function sort(T) table.sort(T, function(a, b) return a.y < b.y end ) end



function formatNumber(number)
   return (string.format("%d", number):reverse():gsub( "(%d%d%d)" , "%1," ):reverse():gsub("^(-?),","%1"))
end
